--MrJackdaw edit!
--Thanks Pengoros - hope this will fix vital bars...
function AddCallback(object, event, callback)
    if (object[event] == nil) then
        object[event] = callback;
    else
        if (type(object[event]) == "table") then
            table.insert(object[event], callback);
        else
            object[event] = {object[event], callback};
        end
    end
    return callback;
end

VitalsWindow = class( Turbine.UI.Window );
VitalsWindow.moraleBar = VitalsBar();
VitalsWindow.powerBar = VitalsBar();

function VitalsWindow:Constructor()
	Turbine.UI.Window.Constructor( self );
	
	self.flashing = false;
	self.flashTime = 0;
	self.flashLimit = 0.33;
	self.locked = Faflaf.Mover.KeyManager.instance:IsLocked();
    self.hidden = Faflaf.Mover.KeyManager.instance:IsHidden();
	
	self:SetBackColor( Turbine.UI.Color( 0.6, 0, 0, 0 ) );
	
	-- Construct the morale bar.
	self.moraleBar:SetParent( self );
	self.moraleBar:SetZOrder( 100 );
	self.moraleBar:SetColor( Turbine.UI.Color( 1, 0.1, 1, 0.1 ) );
	self.moraleBar:SetBuffColor( Turbine.UI.Color( 1, 0.1, 1, 0.2 ) );
	self.moraleBar:SetDebuffColor( Turbine.UI.Color( 1, 0, 0, 0.1 ) );
	self.moraleBar:SetMouseVisible( false );
	
	-- Construct the power bar.
	self.powerBar:SetParent( self );
	self.powerBar:SetZOrder( 100 );
	self.powerBar:SetColor( Turbine.UI.Color( 1, 0.1, 0.5, 1 ) );
	self.powerBar:SetBuffColor( Turbine.UI.Color( 1, 0.1, 0.5, 1 ) );
	self.powerBar:SetDebuffColor( Turbine.UI.Color( 1, 0.1, 0.5, 1 ) );
	self.powerBar:SetMouseVisible( false );

	self.levelLabel = Turbine.UI.Label();
	self.levelLabel:SetParent( self.moraleBar );
	self.levelLabel:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleLeft );
	self.levelLabel:SetForeColor( Turbine.UI.Color(245/255, 222/255, 147/255) );
	self.levelLabel:SetFont( Turbine.UI.Lotro.Font.TrajanPro14 );
	self.levelLabel:SetFontStyle( Turbine.UI.FontStyle.Outline );
	self.levelLabel:SetOutlineColor( Turbine.UI.Color( 0, 0, 0 ) );
	self.levelLabel:SetMouseVisible( false );

	self.nameLabel = Turbine.UI.Label();
	self.nameLabel:SetParent( self.moraleBar );
	self.nameLabel:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleLeft );
	self.nameLabel:SetForeColor( Turbine.UI.Color(245/255, 222/255, 147/255) );
	self.nameLabel:SetFont( Turbine.UI.Lotro.Font.TrajanPro14 );
	self.nameLabel:SetFontStyle( Turbine.UI.FontStyle.Outline );
	self.nameLabel:SetOutlineColor( Turbine.UI.Color( 0, 0, 0 ) );
	self.nameLabel:SetMouseVisible( false );

	-- Hook into the player to receive updates about player morale and power.
	self.player = Turbine.Gameplay.LocalPlayer.GetInstance()
	self.levelLabel:SetText( self.player:GetLevel() );
	self.nameLabel:SetText( self.player:GetName() );
	
	AddCallback(self.player, "NameChanged", function(sender, args) self.nameLabel:SetText( self.player:GetName() ); end)
	AddCallback(self.player, "LevelChanged", function(sender, args) self.levelLabel:SetText( self.player:GetLevel() ); end)
	AddCallback(self.player, "MoraleChanged", function(sender, args) self:UpdateMorale() end)
	AddCallback(self.player, "MaxMoraleChanged", function(sender, args) self:UpdateMorale() end)
	AddCallback(self.player, "BaseMaxMoraleChanged", function(sender, args) self:UpdateMorale() end)
	AddCallback(self.player, "MaxPowerChanged", function(sender, args) self:UpdatePower()  end)
	AddCallback(self.player, "BaseMaxPowerChanged", function(sender, args) self:UpdatePower() end)
	AddCallback(self.player, "PowerChanged", function(sender, args) self:UpdatePower() end)
	AddCallback(self.player, "InCombatChanged", function(sender, args) self:UpdateVisibility(); end)
	
	Faflaf.Mover.KeyManager.instance:AddListener( self );
	-- movement listeners
    self.DragEnd = function(sender)
        local left, top = self:GetPosition();
        settings.WindowLeft, settings.WindowTop = self:GetPosition();
		settings:Save();
    end

	self:UpdateVitals();
	self:PerformLayout();

	if( not self.locked ) then
        self:UiLocked( self.locked );
    end
end

function VitalsWindow:MouseDown( args )
	if( args.Button == Turbine.UI.MouseButton.Right ) then
		vitalsMenu:ShowMenu();
	end
end

function VitalsWindow:UiLocked( locked )
    self.locked = locked;
    if( not self.locked ) then
        self.dragBar = Faflaf.Mover.DragBar( self, "VitalBars" );
        self.dragBar:SetBarOnTop( true );
        self.dragBar:SetBarVisible( not self.hidden );
        self.dragBar:SetDraggable( true );
        
        self:UpdateVisibility();
    else
        if( self.dragBar ~= nil ) then
            self.dragBar:SetDraggable( false );
            self.dragBar:SetTarget( nil );
            self.dragBar = nil;
        end
        
        self:SetBackColor(Turbine.UI.Color(0, 0, 0, 0));
        self:PerformLayout();
    end
end

function VitalsWindow:UiHidden(hidden)
    self.hidden = hidden;
    if( self.dragBar ~= nil ) then
        self.dragBar:SetVisible( not hidden );
    end
    self:UpdateVisibility();
end

function VitalsWindow:UpdateVisibility()
    local visible;
	local opacity;
	local morale = self.player:GetMorale();
	local maxMorale = self.player:GetMaxMorale()
    if( self.hidden ) then
        visible = false;
    elseif( self.locked ) then
        visible = true;
		if( settings.OverrideOpacity and maxMorale > morale ) then
			opacity = settings.InCombatOpacity/10
		elseif( self.player:IsInCombat() ) then
			opacity = settings.InCombatOpacity/10;
		else
			opacity = settings.OutCombatOpacity/10;
		end
		if( settings.FadeOut ) then
			opacity = opacity*(morale/maxMorale);
		end
	else
		visible = false;
	end

	self:SetOpacity( opacity );
    self:SetVisible( visible );
end

function VitalsWindow:UpdateMorale()
	local morale        = self.player:GetMorale();
	local maxMorale     = self.player:GetMaxMorale();
	local baseMaxMorale = self.player:GetBaseMaxMorale();

	self.moraleBar:SetValue( morale );
	self.moraleBar:SetMax( maxMorale );
	self.moraleBar:SetBaseMax( baseMaxMorale );

	local moraleRatio = ( morale / maxMorale );

	local newR = 1 - moraleRatio
	local newG = moraleRatio
	local scaleFactor = 1

	if ( newR < newG ) then
		scaleFactor = 1 / newG
	else
		scaleFactor = 1 / newR
	end

	newR = newR * scaleFactor;
	newG = newG * scaleFactor;

	self.moraleBar:SetColor( Turbine.UI.Color( 1.0, newR, newG, 0.0 ) );

	if ( ( not self.flashing ) and ( moraleRatio < self.flashLimit ) ) then
		self.flashing = true;
		self.flashTime = Turbine.Engine.GetGameTime();
		self:SetWantsUpdates( true );
	end

	if( settings.FadeOut or moraleRatio == 1 ) then self:UpdateVisibility(); end
end

function VitalsWindow:UpdatePower()
	local power = self.player:GetPower()
	self.powerBar:SetValue( power );

	local maxPower = self.player:GetMaxPower()
	self.powerBar:SetMax( maxPower );

	local baseMaxPower = self.player:GetBaseMaxPower()
	self.powerBar:SetBaseMax( baseMaxPower );
end

function VitalsWindow:UpdateVitals()
	self:UpdateMorale();
	self:UpdatePower();
end

function VitalsWindow:Update( args )
	local morale    = self.player:GetMorale();
	local maxMorale = self.player:GetMaxMorale();

	local moraleRatio = ( morale / maxMorale );
	local flashAmount = 0;

	if ( moraleRatio >= self.flashLimit ) then
		self.flashing = false;
		self:SetWantsUpdates( false );
	else
		flashAmount = ( math.sin( 12 * ( Turbine.Engine.GetGameTime() - self.flashTime ) ) + 1 ) / 2;
	end

	local newR = 1 - moraleRatio;
	local newG = moraleRatio;
	local scaleFactor = 1;

	if ( newR < newG ) then
		scaleFactor = 1 / newG
	else
		scaleFactor = 1 / newR
	end

	newR = newR * scaleFactor;
	newG = newG * scaleFactor;

	newR = newR + flashAmount * ( 1 - newR );
	newG = newG + flashAmount * ( 1 - newG );

	self.moraleBar:SetColor( Turbine.UI.Color( 1.0, newR, newG, 0.0 ) );
end

function VitalsWindow:SetMoraleBackgroundImage( image )
	self.moraleBar.fillBar:SetBackground( image );
	self.moraleBar.buffBar:SetBackground( image );
	self.moraleBar.debuffBar:SetBackground( image );
end

function VitalsWindow:SetPowerBackgroundImage( image )
	self.powerBar.fillBar:SetBackground( image );
	self.powerBar.buffBar:SetBackground( image );
	self.powerBar.debuffBar:SetBackground( image );
end

function VitalsWindow:PerformLayout()
	local moraleWidth = settings.PlayerMoraleWidth;
	local moraleHeight = settings.PlayerMoraleHeight;
	local powerWidth = settings.PlayerPowerWidth;
	local powerHeight = settings.PlayerPowerHeight;
	local moraleBackgroundImage;
	local powerBackgroundImage;

	if( settings.PlayerBackground.VariableHeight ) then
		moraleBackgroundImage = settings.PlayerBackground.Image
		powerBackgroundImage = settings.PlayerBackground.Image
	else
		-- if the PlayerBackground does not have a variable height then the image has to have a name with the selected hight in the filename.
		moraleBackgroundImage = string.gsub( settings.PlayerBackground.Image, ".tga", moraleHeight..".tga" );
		powerBackgroundImage = string.gsub( settings.PlayerBackground.Image, ".tga", powerHeight..".tga" );
	end	

	self:SetMoraleBackgroundImage( moraleBackgroundImage );
	self:SetPowerBackgroundImage( powerBackgroundImage );
	self:SetPosition( settings.WindowLeft, settings.WindowTop );
	
	self.moraleBar:SetPosition( 1, 1 );
	self.moraleBar:SetSize( moraleWidth, moraleHeight );
	self.moraleBar:SetShowPercentage( settings.UseMoralePercentage );
	self.moraleBar:SetTextVisible( settings.IsMoraleTextVisible );
	
	self.powerBar:SetSize( powerWidth, powerHeight );
	self.powerBar:SetShowPercentage( settings.UsePowerPercentage );
	self.powerBar:SetTextVisible( settings.IsPowerTextVisible );

	if( settings.PlayerDisplayType == DisplayType.Stacked ) then
		local width;
		if( moraleWidth > powerWidth ) then
			width = moraleWidth+2;
		else
			width = powerWidth+2;
		end
		self.moraleBar:SetVisible(true);
		self.powerBar:SetVisible(true);
		self.powerBar:SetPosition( 1, moraleHeight+2 );
		self:SetSize( width, moraleHeight+powerHeight+3 );	
	elseif( settings.PlayerDisplayType == DisplayType.MoraleOnly ) then
		self.moraleBar:SetVisible(true);
		self.powerBar:SetVisible(false);
		self:SetSize( moraleWidth+2, moraleHeight+2 );
	elseif( settings.PlayerDisplayType == DisplayType.PowerOnly ) then
		self.moraleBar:SetVisible(false);
		self.powerBar:SetVisible(true);
		self.powerBar:SetPosition(1, 1);
		self:SetSize( powerWidth+2, powerHeight+2 );
	else
		local height
		if( moraleHeight > powerHeight ) then
			height = moraleHeight+2;
		else
			height = powerHeight+2;
		end
		self.moraleBar:SetVisible(true);
		self.powerBar:SetVisible(true);
		self.powerBar:SetPosition(moraleWidth+2, 1);
		self:SetSize( moraleWidth+powerWidth+3, height );
	end
	
	self.levelLabel:SetVisible( settings.IsLevelVisible );
	self.levelLabel:SetPosition( 2, -1 );
	self.levelLabel:SetSize( 20, 14 );
	self.levelLabel:SetZOrder( 201 );
	if( self.levelLabel:IsVisible() ) then
		self.nameLabel:SetPosition( (self.levelLabel:GetLeft()+25), -1 );
	else
		self.nameLabel:SetPosition( 2, -1 );
	end
	self.nameLabel:SetVisible( settings.IsNameVisible );
	self.nameLabel:SetSize( 300, 14 );
	self.nameLabel:SetZOrder( 201 );

	self.moraleBar:Update();
	self.powerBar:Update();

	self:UpdateVisibility();
end